import os
from os.path import expanduser
import argparse
import logging
from googleapiclient import discovery
from oauth2client.client import GoogleCredentials
#note: we need to know where to bring in the token_api / tokenutils.
#from token_api import Token_api

logger = logging.getLogger('HA')
logger.setLevel(logging.DEBUG)

class csr_ha():
    def __init__(self):
        self.credentials = GoogleCredentials.get_application_default()
        self.service = discovery.build('compute', 'v1', credentials=self.credentials)
        self.home = expanduser("~")
        self.cert_file = "/etc/ssl/certs/ca-bundle.trust.crt"
        self.event_dir = "%s/gcp/HA/events" % self.home
        self.event_dir = "%s/gcp" % self.home
        if not os.path.exists(self.event_dir):
            os.mkdir(self.event_dir)
        # self.set_response_file = self.event_dir + '/routeTableSetRsp'
        # self.get_response_file = self.event_dir + '/routeTableGetRsp'
        # note: do we need these ?  will we install via --user ?
        # sys.path.append(self.home + '/.local/lib/python2.7/site-packages/csr_ha/client_api')
        # sys.path.append(self.home + '.local/lib/python2.7/site-packages/csr_ha/server')
        self.cloudname = 'gcp'
        self.route_priority = 65535
        self.route_name = "csr-ha-verify-route"

    def get_sock_filename(self):
        return "%s/%s/HA/sock_file" % (self.home, self.cloudname)

    def get_node_filename(self):
        return '%s/%s/HA/node_file' % (self.home, self.cloudname)

    def get_route_by_routename(self, node, event_type=None):
        try:
            request = self.service.routes().get(project=node['project'], route=node['routeName'])
            response = request.execute()
            if event_type == 'verify':
                # with open(self.get_response_file, 'w') as resp_fh:
                #     json.dump(response, resp_fh, sort_keys=True, indent=4)
                logger.info("Read route successfully")
        except Exception as e:
            logger.exception("Route GET request failed with error: %s" % e)
            # with open(self.get_response_file, 'w') as resp_fh:
            #     json.dump(str(e), resp_fh)
            return None
        return response

    def get_vpc(self, node):
        route = self.get_route_by_routename(node)
        if route:
            try:
                network = route['network']
                vpc = network.split('/')[-1]
                logger.info("The routes are in VPC: %s" % vpc)
            except Exception as e:
                logger.exception("Error in fetching vpc from route is %s" % e)
                vpc = None
            return vpc

    def delete_route(self, node, route_name = None):
        if route_name:
            r_name = route_name
        else:
            r_name = node['routeName']
        try:
            request = self.service.routes().delete(project=node['project'], route=r_name)
            response = request.execute()
        except Exception as e:
            logger.exception("Error is deleting route is %s " % e)
            return None
        return response

    def insert_route(self, node, newNextHop=None, route_priority=None, route_name = None):
        # Insert new route
        if route_priority:
            prior = route_priority
        else:
            prior = node['hopPriority']
        if route_name:
            r_name = route_name
        else:
            r_name = node['routeName']

        if newNextHop:
            nextHop = newNextHop
        else:
            nextHop = node['nextHop']

        route_body = {
            "destRange": node['route'],
            "name": r_name,
            "network": "https://www.googleapis.com/compute/beta/projects/{}/global/networks/{}".format(
                node['project'], node['vpc']),
            "priority": prior,
            "tags": node['tag'],
            "nextHopIp": nextHop
        }
        logger.info("Inserting route: ")
        for key, value in route_body.items():
            logger.info(key, ":", value)
        try:
            request = self.service.routes().insert(project=node['project'], body=route_body)
            response = request.execute()
        except Exception as e:
            logger.exception("Route Not inserted. Error is %s" % e)

    def set_one_route(self, node, event_type, route_table=None, event_file=None):
        if event_type == 'verify':
            logger.info("Evaluating single route for event type %s" % event_type)
            route = self.get_route_by_routename(node, event_type)
            newNextHop = route['nextHopIp']
            response = self.insert_route(node, newNextHop, self.route_priority, self.route_name)
            response = self.delete_route(node, self.route_name)

        elif event_type == 'revert':
            #do something
            route = self.get_route_by_routename(node, event_type)
            if route:
                if event_file:
                    os_command = "rm \"%s\"" % event_file
                    os.system(os_command)
                logger.info("Route already exists. Nothing to do.")
            else:
                response = self.insert_route(node)

        elif event_type == "peerFail":
            peer_route_name = node['peerRouteName']
            response = self.delete_route(node, peer_route_name)

        else:
            logger.error("Invalid event type %s in set_route_table" % event_type)

        return

    def set_route_table(self, node, event_type, route_table, event_file):
        self.set_one_route(node, event_type, route_table,  event_file)
        return

    def verify_node(self, node, event_type):
        if event_type == 'verify':
            node_verified = True
            if (not (node.has_key('project'))):
                logger.info("Missing required parameter -g project")
                node_verified = False
            if (not (node.has_key('vpc'))):
                logger.info("Missing required parameter -v vpc")
                node_verified = False
            if (not (node.has_key('route'))):
                logger.info("Missing required parameter -d route")
                node_verified = False
            if (not (node.has_key('routeName'))):
                logger.info("Missing required parameter -r routeName")
                node_verified = False
            if (not (node.has_key('peerRouteName'))):
                logger.info("Missing required parameter -l peerRouteName")
                node_verified = False
            if (not (node.has_key('nextHop'))):
                logger.info("Missing required parameter -n nextHopIpAddress")
                node_verified = False
            if (not (node.has_key('tag'))):
                logger.info("Missing required parameter -t tag")
                node_verified = False
            if (not (node.has_key('hopPriority'))):
                logger.info("Missing required parameter -P hopPriority")
                node_verified = False

            if node_verified:
                logger.info("All required parameters have been provided")
                return 'OK'
            else:
                return 'ERR1'

        else:
            if ((node.has_key('index')) and
                    (node.has_key('cloud')) and
                    (node.has_key('project')) and
                    (node.has_key('vpc')) and
                    (node.has_key('route')) and
                    (node.has_key('routeName')) and
                    (node.has_key('peerRouteName')) and
                    (node.has_key('tag')) and
                    (node.has_key('hopPriority')) and
                    (node.has_key('nextHop'))):
                return 'OK'
            else:
                logger.info("verify_node: missing required parameter")
                return "ERR1"

    def build_command_dict(self, cmd_list):

        cmd_dict = {}
        i = 1
        while i < len(cmd_list):
            if cmd_list[i] == '-i':
                keyword = "index"
            elif cmd_list[i] == '-p':
                keyword = "cloud"
            elif cmd_list[i] == '-g':
                keyword = "project"
            elif cmd_list[i] == '-v':
                keyword = "vpc"
            elif cmd_list[i] == '-d':
                keyword = "route"
            elif cmd_list[i] == '-r':
                keyword = "routeName"
            elif cmd_list[i] == '-l':
                keyword = "peerRouteName"
            elif cmd_list[i] == '-n':
                keyword = "nextHop"
            elif cmd_list[i] == '-m':
                keyword = "mode"
            elif cmd_list[i] == '-t':
                keyword = "tag"
            elif cmd_list[i] == '-P':
                keyword = "hopPriority"
            elif cmd_list[i] == '-c':
                keyword = "command"
            elif cmd_list[i] == '-e':
                keyword = "event"
            elif cmd_list[i] == '':
                break
            else:
                logger.error("Invalid command format %s" % cmd_list[i])
                return {}
            value = cmd_list[i + 1]
            i = i + 2
            cmd_dict[keyword] = value
        #        c.logger.info("Added keyword %s with value %s" % (keyword, value)
        return cmd_dict

    def create_node(self, params):
        node = params
        try:
            vpc = node['vpc']
        except KeyError:
            vpc = self.get_vpc(node)
            if vpc != '':
                node['vpc'] = vpc
        #
        # try:
        #     nextHop = node['nextHop']
        # except KeyError:
        #     nextHop = nodeTable.get_private_ip()
        #     if nextHop != '':
        #         node['nextHop'] = nextHop

        return node

    def check_clear_param(self, index, old_params):

        keyword = " "
        if old_params[index] == '-g':
            keyword = "project"
        if old_params[index] == '-v':
            keyword = "vpc"
        elif old_params[index] == '-d':
            keyword = "route"
        elif old_params[index] == '-t':
            keyword = "tag"
        elif old_params[index] == '-r':
            keyword = "routeName"
        elif old_params[index] == '-l':
            keyword = "peerRouteName"
        elif old_params[index] == '-n':
            keyword = "nextHop"
        elif old_params[index] == '-m':
            keyword = "mode"
        elif old_params[index] == '-P':
            keyword = "hopPriority"
        else:
            logger.info("Invalid parameter format %s" % old_params[index])

        return keyword

    def clear_param_parser(self):

        parser = argparse.ArgumentParser(description="CSR HA Clear Node")
        parser._action_groups.pop()
        required = parser.add_argument_group('required arguments')
        optional = parser.add_argument_group('optional arguments')

        required.add_argument('-i', nargs='?', help='index', default=None)
        optional.add_argument('-g', help='to clear the project id', default=None)
        optional.add_argument('-d', help='to clear the route', default=None)
        optional.add_argument('-r', help='to clear the routeName', default=None)
        optional.add_argument('-l', help='to clear the peerRouteName', default=None)
        optional.add_argument('-n', help='to clear the nextHopIpAddress', default=None)
        optional.add_argument('-m', help='to clear the mode', default=None)
        optional.add_argument('-c', help='command', default=None)
        optional.add_argument('-P', help='to clear the hopPriority', default=None)
        optional.add_argument('-t', help='to clear the tag', default=None)

        return parser

    def create_node_parser(self):

        parser = argparse.ArgumentParser(description="CSR HA Create Node")
        parser._action_groups.pop()
        required = parser.add_argument_group('required arguments')
        optional = parser.add_argument_group('optional arguments')

        required.add_argument('-i', nargs='?', help='<routeIndex>       (1..256)', default=None, required=True)
        required.add_argument('-g', help='<project>', default=None)
        required.add_argument('-r', help='<routeName>', default=None, required=True)
        required.add_argument('-l', help='<peerRouteName>', default=None, required=True)
        required.add_argument('-n', help='<nextHopIpAddress>', default=None, required=True)
        required.add_argument('-d', help='<route>', default=None)
        optional.add_argument('-p', help='<cloud_provider>   {gcp}', choices=['gcp'],
                              default=None, required=True)
        optional.add_argument('-P', help='<hopPriority>', default=None)
        optional.add_argument('-t', help='<tag>', default=None)
        optional.add_argument('-m', help='<mode>  {primary | secondary}', choices=['primary', 'secondary'],
                              default=None)

        return parser

    def set_node_parser(cmd):

        parser = argparse.ArgumentParser(description="CSR HA Set Node")
        parser._action_groups.pop()
        required = parser.add_argument_group('required arguments')
        optional = parser.add_argument_group('optional arguments')

        required.add_argument('-i', nargs='?', help='<routeIndex>', type=int, default=None, required=True)
        optional.add_argument('-g', help='to set the projectid', default=None)
        optional.add_argument('-r', help='to set the routeName', default=None)
        optional.add_argument('-l', help='to set the peerRouteName', default=None)
        optional.add_argument('-d', help='to set the route', default=None)
        optional.add_argument('-n', help='to set the nextHopIpAddress', default=None)
        optional.add_argument('-m', help='to set the mode', default=None)
        optional.add_argument('-c', help='command', default=None)
        optional.add_argument('-t', help='to set the tag', default=None)
        optional.add_argument('-P', help='to set the hopPriority', default=None)

        return parser